@@ -53,6 +53,7 @@ extern unsigned char reuse_initrd;
 /* Used for enabling printing message from purgatory code
  * Only has implemented for PPC64 */
 int my_debug;
+int dt_no_old_root;
 
 /* This provides the behaviour of hte existing ppc64 implementation */
 static void pad_structure_block(size_t len) {
@@ -122,16 +123,17 @@ static void checkprop(char *name, unsigned *data, int len)
 	else if (!strcmp(name, "rtas-size") ||
 			!strcmp(name, "linux,tce-size"))
 		size = be32_to_cpu(*data);
-	else if (reuse_initrd && !strcmp(name, "linux,initrd-start"))
+	else if (reuse_initrd && !strcmp(name, "linux,initrd-start")) {
 		if (len == 8)
 			base = be64_to_cpu(*(unsigned long long *) data);
 		else
 			base = be32_to_cpu(*data);
-	else if (reuse_initrd && !strcmp(name, "linux,initrd-end"))
+	} else if (reuse_initrd && !strcmp(name, "linux,initrd-end")) {
 		if (len == 8)
 			end = be64_to_cpu(*(unsigned long long *) data);
 		else
 			end = be32_to_cpu(*data);
+	}
 
 	if (size && end)
 		die("unrecoverable error: size and end set at same time\n");
@@ -321,7 +323,7 @@ static void add_usable_mem_property(int fd, size_t len)
 
 	ranges = malloc(ranges_size * sizeof(*ranges));
 	if (!ranges)
-		die("unrecoverable error: can't alloc %d bytes for ranges.\n",
+		die("unrecoverable error: can't alloc %zu bytes for ranges.\n",
 		    ranges_size * sizeof(*ranges));
 
 	for (range = 0; range < usablemem_rgns.size; range++) {
@@ -344,7 +346,7 @@ static void add_usable_mem_property(int fd, size_t len)
 						 sizeof(*ranges));
 				if (!ranges)
 					die("unrecoverable error: can't realloc"
-					    "%d bytes for ranges.\n",
+					    "%zu bytes for ranges.\n",
 					    ranges_size*sizeof(*ranges));
 			}
 			ranges[rlen++] = cpu_to_be64(loc_base);
@@ -510,6 +512,37 @@ static int comparefunc(const struct dirent **dentry1,
 	return strcmp(str1, str2);
 }
 
+/* grab root= from the old command line */
+static void dt_copy_old_root_param(void)
+{
+	FILE *fp;
+	char filename[MAXPATH];
+	char *last_cmdline = NULL;
+	char *p, *old_param;
+	size_t len = 0;
+
+	strcpy(filename, pathname);
+	strcat(filename, "bootargs");
+	fp = fopen(filename, "r");
+	if (fp) {
+		if (getline(&last_cmdline, &len, fp) == -1)
+			die("unable to read %s\n", filename);
+
+		p = strstr(last_cmdline, "root=");
+		if (p) {
+			old_param = strtok(p, " ");
+			len = strlen(local_cmdline);
+			if (len != 0)
+				strcat(local_cmdline, " ");
+			strcat(local_cmdline, old_param);
+		}
+	}
+	if (last_cmdline)
+		free(last_cmdline);
+
+	fclose(fp);
+}
+
 /*
  * put a node (directory) in the property structure.  first properties
  * then children.
@@ -578,8 +611,11 @@ static void putnode(void)
 		reserve(initrd_base, initrd_size);
 	}
 
-	/* Add cmdline to the second kernel.  Check to see if the new
-	 * cmdline has a root=.  If not, use the old root= cmdline.  */
+	/*
+	 * Add cmdline to the second kernel. Use the old root= cmdline if there
+	 * is no root= in the new command line and there's no --dt-no-old-root
+	 * option being used.
+	 */
 	if (!strcmp(basename,"chosen/")) {
 		size_t result;
 		size_t cmd_len = 0;
@@ -597,30 +633,9 @@ static void putnode(void)
 			param = strstr(local_cmdline, "root=");
 		}
 
-		/* ... if not, grab root= from the old command line */
-		if (!param) {
-			FILE *fp;
-			char *last_cmdline = NULL;
-			char *old_param;
+		if (!param && !dt_no_old_root)
+			dt_copy_old_root_param();
 
-			strcpy(filename, pathname);
-			strcat(filename, "bootargs");
-			fp = fopen(filename, "r");
-			if (fp) {
-				if (getline(&last_cmdline, &cmd_len, fp) == -1)
-					die("unable to read %s\n", filename);
-
-				param = strstr(last_cmdline, "root=");
-				if (param) {
-					old_param = strtok(param, " ");
-					if (cmd_len != 0)
-						strcat(local_cmdline, " ");
-					strcat(local_cmdline, old_param);
-				}
-			}
-			if (last_cmdline)
-				free(last_cmdline);
-		}
 		strcat(local_cmdline, " ");
 		cmd_len = strlen(local_cmdline);
 		cmd_len = cmd_len + 1;
